SystemOrganization addCategory: #MkSack!
SystemOrganization addCategory: #'MkSack-Tests'!

Object subclass: #SackRoll
	instanceVariableNames: 'dice score taken remaining'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkSack'!

!SackRoll classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 10/17/2010 19:23'!
atRandom: numberOfDice
	^ self basicNew initalize: ((1 to: numberOfDice) collect: [:ignore|6 atRandom]).
! !

!SackRoll classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 10/17/2010 19:23'!
new: aRoll
	^ self basicNew initalize: aRoll! !

!SackRoll methodsFor: 'converting' stamp: 'MathiasKegelmann 10/19/2010 23:15'!
asString
	^ super asString, '(', dice asString, ')'! !

!SackRoll methodsFor: 'testing' stamp: 'MathiasKegelmann 10/19/2010 18:25'!
combinesToStraightWith: rest
	|set|
	set := dice asSet.
	set addAll: rest.
	^ set size = (dice size + rest size)! !

!SackRoll methodsFor: 'evaluating' stamp: 'MathiasKegelmann 10/19/2010 23:21'!
evaluate
	score := 0.
	taken := OrderedCollection new.
	remaining := dice asOrderedCollection.
	self testTripleOnes.
	self testOrdinaryTriples.
	self evaluateOnes.
	self evaluateFives.! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/19/2010 23:33'!
evaluate: n withValue: value
	|hits|
	hits := remaining select: [:x | x = n].
	taken addAll: hits.
	score := score + (value * hits size).
	remaining removeAll: hits.! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/17/2010 21:06'!
evaluateFives
	self evaluate: 5 withValue: 50.! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/17/2010 21:11'!
evaluateOnes
	self evaluate: 1 withValue: 100.! !

!SackRoll methodsFor: 'initialize-release' stamp: 'MathiasKegelmann 10/18/2010 09:33'!
initalize: anIntCollection
	dice := anIntCollection copy.
	self evaluate.! !

!SackRoll methodsFor: 'testing' stamp: 'MathiasKegelmann 10/18/2010 09:38'!
isForcedContinue
	^ remaining isEmpty ! !

!SackRoll methodsFor: 'testing' stamp: 'MathiasKegelmann 10/18/2010 09:12'!
isSack
	^ self score = 0! !

!SackRoll methodsFor: 'accessing' stamp: 'MathiasKegelmann 10/18/2010 09:31'!
remaining
	^ remaining copy.! !

!SackRoll methodsFor: 'accessing' stamp: 'MathiasKegelmann 10/17/2010 21:04'!
score
	^ score! !

!SackRoll methodsFor: 'accessing' stamp: 'MathiasKegelmann 10/17/2010 19:16'!
size
	^ dice size! !

!SackRoll methodsFor: 'accessing' stamp: 'MathiasKegelmann 10/18/2010 09:31'!
taken 
	^ taken copy.! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/19/2010 23:28'!
testOrdinaryTriples
	(2 to: 6) do: [:i | self triples: i withValue: i * 100]
! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/19/2010 23:26'!
testTripleOnes
	self triples: 1 withValue: 1000! !

!SackRoll methodsFor: 'private-evaluating' stamp: 'MathiasKegelmann 10/19/2010 23:33'!
triples: n withValue: value
	|hits k|
	hits := remaining select: [:x | x = n].
	k := hits size.
	k >= 3 ifTrue: [
		taken addAll: hits.
		score := score + value * (1 << (k - 3)).
		remaining removeAll: hits]! !

Object subclass: #SackRollout
	instanceVariableNames: 'initialTake rounds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkSack'!

!SackRollout classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 10/20/2010 08:42'!
for: anInitialTake
	^ self for: anInitialTake with: 10000! !

!SackRollout classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 10/20/2010 08:34'!
for: anInitialTake with: nRounds
	^ self basicNew initializeFor: anInitialTake with: nRounds ! !

!SackRollout classSide methodsFor: 'running' stamp: 'MathiasKegelmann 10/20/2010 13:21'!
stats
	Transcript tab; show: ((SackRollout for: #()) run); cr.
	(1 to: 6) do: [:x | Transcript show: x; tab; show: ((SackRollout for: {x}) run); cr] 
		displayingProgress: 'Singletons'.
	((1 to: 6) asSet choose: 2) do: [:s | Transcript show: s; tab; show: ((SackRollout for: s) run); cr]
		displayingProgress: 'Pairs'.
	Transcript show: '11'; tab; show: ((SackRollout for: #(1 1)) run); cr.
	Transcript show: '15'; tab; show: ((SackRollout for: #(1 5)) run); cr.
	Transcript show: '55'; tab; show: ((SackRollout for: #(5 5)) run); cr.
	((1 to: 6) asSet choose: 3) do: [:s | Transcript show: s; tab; show: ((SackRollout for: s) run); cr]
		displayingProgress: 'Triples'.
! !

!SackRollout methodsFor: 'initialization' stamp: 'MathiasKegelmann 10/20/2010 08:35'!
initializeFor: anInitialTake with: nRounds 
	initialTake := anInitialTake.
	rounds := nRounds.
	! !

!SackRollout methodsFor: 'running' stamp: 'MathiasKegelmann 10/20/2010 08:40'!
run
	"Do a roll out for 'rounds'' number of rounds and return the result."
	|total|
	total := 0.
	rounds timesRepeat: [ total := total + (SackRound new: initialTake) run ].
	^ total asFloat / rounds
		! !

Object subclass: #SackRound
	instanceVariableNames: 'dice baseScore currentScore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkSack'!

!SackRound classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 10/19/2010 18:27'!
new: initialTake
	^ self basicNew initialize: initialTake ! !

!SackRound methodsFor: 'private' stamp: 'MathiasKegelmann 10/19/2010 23:14'!
continue
	baseScore := baseScore + currentScore.
	currentScore := 0.
	dice := Bag new.! !

!SackRound methodsFor: 'initialization' stamp: 'MathiasKegelmann 10/19/2010 18:48'!
initialize: initialTake
	baseScore := 0.
	dice := initialTake asBag.
	currentScore := (SackRoll new: initialTake) score.
	! !

!SackRound methodsFor: 'running' stamp: 'MathiasKegelmann 10/20/2010 08:42'!
run
	"Keep stepping till the score reaches 350 or we have a sack. Then return the score."
	[true] whileTrue: [ |roll| 
		roll := SackRoll atRandom: 5 - dice size.
		"Transcript show: roll; cr."
		(roll combinesToStraightWith: dice) ifTrue: [
			currentScore := 1500.
			self continue]
		 ifFalse: [ |score| 
			roll isSack ifTrue: [ ^0 ].
			dice addAll: roll taken.
			currentScore := currentScore + roll score.
			roll isForcedContinue ifTrue: [ self continue	]
			ifFalse:  [
				score := baseScore + currentScore.
				score >= 350 ifTrue: [^score]
			]
		]
	]
	! !

!Collection methodsFor: '*MkSack' stamp: 'MathiasKegelmann 10/20/2010 12:55'!
choose: n
	"Return the Bag of all n element subcollections"
	(n = 0) ifTrue: [^ self class with: self class new].
	^ self inject: self class new into: [:total :x |
		total, (((self copyWithout: x) choose: n - 1) collect: [:each | each, {x}]) 
	]
! !

TestCase subclass: #SackRollTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkSack-Tests'!

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
roll: aRoll
	^ SackRoll new: aRoll! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:39'!
testAllOnesAndFivesAreForcedContinue
	self assert: (self roll: #(1 5 5 1)) isForcedContinue ! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/17/2010 19:19'!
testAtRandom
	(1 to: 5) do: [:n|
		self assert: n equals: (SackRoll atRandom: n) size ]! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 18:22'!
testCombinesToOneBasedStraight
	self assert: ((self roll: #(1 5)) combinesToStraightWith: #(4 2 3))! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 18:22'!
testCombinesToSixBasedStraight
	self assert: ((self roll: #(6 2 5)) combinesToStraightWith: #(4 3))! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:39'!
testDoublesAreNotForcedContinue
	self deny:  (self roll: #(2 2 3 3 4 4 6 6)) isForcedContinue! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:23'!
testDoublesAreNotTaken
	self assert:  (self roll: #(2 2 3 3 4 4 6 6)) taken isEmpty ! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testDoublesAreSack
	self assert:  (self roll: #(2 2 3 3 4 4 6 6)) isSack.! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testDoublesCountNothing
	self assert: 0 equals: (self roll: #(2 2 3 3 4 4 6 6)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:35'!
testDoublesRemain
	self assert:  (#(2 2 3 3 4 4 6 6) hasEqualElements: (self roll: #(2 2 3 3 4 4 6 6)) remaining)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testFiveCountsFifty
	self assert: 50 equals: (self roll: #(5)) score ! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testFiveImpliesNoSack
	self deny: (self roll: #(5)) isSack ! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testFivePlusNoiseCountsFifty
	self assert: 50 equals:  (self roll: #(2 5 3 3)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:25'!
testFivePlusNoiseImpliesFivesAreTaken
	self assert: (#(5 5) hasEqualElements:  (self roll: #(2 5 3 5 3)) taken)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:35'!
testFivePlusNoiseRemainsNoise
	self assert: (#(2 3 3) hasEqualElements: (self roll: #(2 5 3 3)) remaining)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 18:23'!
testNoStraight1
	self deny: ((self roll: #(2 2)) combinesToStraightWith: #(1 3 4))! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 18:23'!
testNoStraight2
	self deny: ((self roll: #(2 3)) combinesToStraightWith: #(1 3 3))! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 18:23'!
testNoStraight3
	self deny: ((self roll: #(2 3)) combinesToStraightWith: #(1 3 4))! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testOneCountsOneHundred
	self assert: 100 equals: (self roll: #(1)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testOnePlusNoiseCountsOneHundred
	self assert: 100 equals: (self roll: #(2 1 3 3)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testScoreForOnesAndFives
	self assert: 300 equals: (self roll: #(1 2 5 5 1)) score ! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testScoreForOnesAndFives2
	self assert: 250 equals: (self roll: #(1 2 5 2 1)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:35'!
testScoreForOnesAndFivesNoiseRemains
	self assert: (#(2) hasEqualElements: (self roll: #(1 2 5 5 1)) remaining)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testSingleCountsNothing
	self assert:0 equals:  (self roll: #(4)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:37'!
testTakenForOnesAndFivesWithNoise
	self assert: (#(1 1 5 5) hasEqualElements: (self roll: #(1 2 5 5 1)) taken)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:35'!
testTripleMixed
	self assert: 550 equals: (self roll: #(1 5 4 4 4)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:34'!
testTripleScore1000
	self assert: 1000 equals: (self roll: #(2 1 1 1 6)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:34'!
testTripleScore2000
	self assert: 2000 equals: (self roll: #(1 1 1 1 6)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:34'!
testTripleScore4000
	self assert: 4000 equals: (self roll: #(1 1 1 1 1)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:20'!
testTripleScore5
	self assert: 500 equals: (self roll: #(2 5 5 5 6)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:20'!
testTripleScore6
	self assert: 600 equals: (self roll: #(2 6 6 6 4)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:35'!
testTripleScoreFiveFours
	self assert: 1600 equals: (self roll: #(4 4 4 4 4)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/19/2010 23:35'!
testTripleScoreFourFours
	self assert: 800 equals: (self roll: #(2 4 4 4 4)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testTwoFivesCountsOneHundred
	self assert: 100 equals:  (self roll: #(5 5)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testTwoFivesPlusNoiseCountsOneHundred
	self assert: 100 equals: (self roll: #(5 5 6 6 3)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:36'!
testTwoFivesPlusNoiseRemainsNoise
	self assert: (#(6 6 3) hasEqualElements: (self roll: #(5 5 6 6 3)) remaining)! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:13'!
testTwoOnesPlusNoiseCountsTwoHundred
	self assert: 200 equals: (self roll: #(1 1 6 6 3)) score! !

!SackRollTest methodsFor: 'tests' stamp: 'MathiasKegelmann 10/18/2010 09:36'!
testTwoOnesPlusNoiseRemainsNoise
	self assert: (#(6 6 3) hasEqualElements: (self roll: #(1 1 6 6 3)) remaining)! !
